function[Mat1Fin, Mat2Fin, Mat3Fin, Mat4Fin] = hpp2(Mat1, Mat2, Mat3, Mat4, Mat2Temp, Mat4Temp) % HPP Hardy, Pomeau, Pazzis ; vertical

m = size(Mat2,1); n = size(Mat2,2);
Mat2Fin = zeros(m, n); Mat4Fin = zeros(m, n);
Mat1Temp = zeros(m, n); Mat3Temp = zeros(m, n); % Matrices temporaires
                                                % stockant les cellules qui
                                                % ont changé de vecteur,
                                                % mais qui se meuvent selon
                                                % le vecteur précédent

Mat2Right = circshift(Mat2,[0,2]);
Mat4Left = circshift(Mat4,[0,-2]);
Mat1Left = circshift(Mat1,[-1,-1]); Mat1Right = circshift(Mat1,[-1,1]);
Mat3Left = circshift(Mat3,[1,-1]); Mat3Right = circshift(Mat3,[1,1]);

for i=1:m
    for j=1:n
        if Mat2(i,j) == 1 % S'il y a vecteur droit, test collisions
            if Mat4Left(i,j) == 0 % S'il n'y a pas de collision frontale...
                Mat2Fin(i,j) = 1; % ...la particule garde son vecteur
            end
            if Mat4Left(i,j) == 1 % Sinon...
                Mat1Temp(i,j) = 1; % ...son vecteur change
            end
        end
        if Mat4(i,j) == 1 % S'il y a vecteur gauche, test collisions
            if Mat2Right(i,j) == 0 % S'il n'y a pas de collision frontale...
                Mat4Fin(i,j) = 1; % ...la particule garde son vecteur
            end
            if Mat2Right(i,j) == 1 % Sinon...
                Mat3Temp(i,j) = 1; % ...son vecteur change
            end
        end
    end
end

Mat3Temp = circshift(Mat3Temp,[0,-1]);
Mat1Temp = circshift(Mat1Temp,[0,1]);

Mat2Fin = circshift(Mat2Fin,[0,1])+Mat2Temp;
Mat4Fin = circshift(Mat4Fin,[0,-1])+Mat4Temp;

for i=1:m % Lorsqu'il y a deux particules au même endroit avec le même vecteur
    for j=1:n
        if Mat2Fin(i,j) == 2 % droite
            Mat1Temp(i,j) = 1; % devient haut
            Mat2Fin(i,j) = 1;
        end
        if Mat4Fin(i,j) == 2 % gauche
            Mat3Temp(i,j) = 1; % devient bas
            Mat4Fin(i,j) = 1;
        end
    end
end

Mat1Fin = Mat1+Mat1Temp; % Pour ajouter aux matrices verticaux les
Mat3Fin = Mat3+Mat3Temp; % nouvelles particules après collision hoizontale

for i=1:m % Lorsqu'il y a deux particules au même endroit avec le même vecteur
    for j=1:n
        if Mat1Fin(i,j) == 2 % haut
            Mat2Fin(i,j) = 1; % devient droit
            Mat1Fin(i,j) = 1;
        end
        if Mat3Fin(i,j) == 2 % bas
            Mat4Fin(i,j) = 1; % devient gauche
            Mat3Fin(i,j) = 1;
        end
    end
end